@@ -11,7 +11,6 @@ import subprocess
 import warnings
 import itertools
 warnings.filterwarnings('ignore')
-import pdb
 
 
 # Given a result from the aligning with Kalign, trims residues from
@@ -49,15 +48,29 @@ def invoke_kalign(input_file, output_file):
     # and fails to detect its type.
     # Pass in a fake pty to work around this behavior, which is appropriate
     # for command-line usage but doesn't work in a headless environment.
-    fakepty, _ = pty.openpty()
+    fakepty, alsofakepty = pty.openpty()
     command = shlex.split('kalign -i %s -o %s' % (input_file, output_file))
-    kaligner = subprocess.Popen(command, stderr=subprocess.PIPE, stdout=subprocess.PIPE,
-                                stdin=fakepty)
+    kaligner = subprocess.Popen(command, stderr=subprocess.PIPE,
+                                stdout=subprocess.PIPE, stdin=fakepty)
     kaligner.wait()
     stdout, stderr = kaligner.communicate()
+    os.close(fakepty)
+    os.close(alsofakepty)
+    # If Kalign fails try again with MAFFT
     if kaligner.returncode != 0:
-        print(stderr, file=sys.stderr)
-        exit(kaligner.returncode)
+        with open(output_file, 'w') as out_fh:
+            command = shlex.split('mafft %s' % input_file)
+            mafft = subprocess.Popen(command, stdout=out_fh)
+            mafft.wait()
+            _, stderr = mafft.communicate()
+        if mafft.returncode != 0:
+            sys.stderr.write('Error running mafft')
+            sys.stderr.write(stderr.decode('utf-8'))
+            return 1
+        else:
+            return 0
+    else:
+        return 0
 
 
 # Reinserts '*' at end of gapped alignment to make finding the end of
@@ -77,7 +90,7 @@ def reinsert_star(seq, gapped_seq):
 
 # Given a single gene's protein sequence and its entire DNA genome,
 # finds the DNA seqeunce that corresponds with the given gene.
-def extract_DNA_seq(seq, trans):
+def extract_DNA_seq(seq, trans, shift):
     Pseq = seq.translate()
     check = 0
     for i in range(len(Pseq)):
@@ -85,7 +98,7 @@ def extract_DNA_seq(seq, trans):
             check = 1
             break
     if check == 1:
-        return seq[i*3:(i+len(trans))*3]
+        return seq[i*3+shift:(i+len(trans))*3+shift]
     else:
         return 1
 
@@ -95,7 +108,7 @@ def extract_DNA_seq(seq, trans):
 def join_extract_DNA_seq(seq, homologs):
     Dseqs = []
     for homolog in homologs:
-        frame = homolog[1]
+        shift = homolog[1]
         Pseq = seq[homolog[1]:].translate()
         check = 0
         for i in range(len(Pseq)):
@@ -103,7 +116,7 @@ def join_extract_DNA_seq(seq, homologs):
                 check = 1
                 break
         if check == 1:
-            Dseqs.append(seq[i*3+frame:(i+len(homolog[0]))*3+frame])
+            Dseqs.append(seq[i*3+shift:(i+len(homolog[0]))*3+shift])
         else:
             return 1
     return Dseqs[0]+Dseqs[1]
@@ -146,7 +159,9 @@ def find_homologs(seq1, seq2):
         stops = seq2[i:].translate()
         to_align = [SeqRecord(seq1, id='Seq1'), SeqRecord(trans0, id='Seq2')]
         SeqIO.write(to_align, 'tmp.fasta', 'fasta')
-        invoke_kalign('tmp.fasta', 'out.fasta')
+        kresult = invoke_kalign('tmp.fasta', 'out.fasta')
+        if kresult == 1:
+            return 1
         alignments = []
         for record in SeqIO.parse('out.fasta', 'fasta'):
             alignments.append(record.seq)
@@ -179,7 +194,9 @@ def join_find_homologs(seqs, seq2):
             to_align = [SeqRecord(seq, id='Seq1'),
                         SeqRecord(trans0, id='Seq2')]
             SeqIO.write(to_align, 'tmp.fasta', 'fasta')
-            invoke_kalign('tmp.fasta', 'out.fasta')
+            kresult = invoke_kalign('tmp.fasta', 'out.fasta')
+            if kresult == 1:
+                return 1
             alignments = []
             for record in SeqIO.parse('out.fasta', 'fasta'):
                 alignments.append(record.seq)
@@ -276,7 +293,7 @@ def create_lists(reads, seq, og_seqs, join):
             if join == 0:
                 seqs.append(result[0])
                 og_seqs[record.id] = extract_DNA_seq(record.seq[result[1]:],
-                                                     seqs[-1])
+                                                     seqs[-1], result[1])
             elif join == 1:
                 seqs.append(result[0][0]+result[1][0])
                 og_seqs[record.id] = join_extract_DNA_seq(record.seq, result)
@@ -366,7 +383,7 @@ def compressor(seqs, names, ids, og_seqs):
             new_seqs.append(seqs[ids.index(same[0])])
             new_og_seqs[new_ids[-1]] = og_seqs[new_ids[-1]]
         else:
-            new_ids.append('MultiSeq_'+str(count)+'_('+str(len(same))+')')
+            new_ids.append('MultiSeq'+str(count)+'_'+str(len(same)))
             count += 1
             name = ''
             for s in same:
@@ -395,8 +412,8 @@ def genome_mode(reference, reads, start, end, compress):
         idd = record.id
         name = record.description
         if isinstance(start, tuple):
-            seq = (record.seq[start[0] % 3:].translate()[start[0]//3:end[0]//3],
-                   record.seq[start[1] % 3:].translate()[start[1]//3:end[1]//3])
+            seq = (record.seq[start[0] % 3:].translate(table=1)[start[0]//3:end[0]//3],
+                   record.seq[start[1] % 3:].translate(table=1)[start[1]//3:end[1]//3])
             og_seqs[record.id] = record.seq[start[0]:end[0]]+record.seq[start[1]:end[1]]
         else:
             seq = record.seq[start % 3:].translate()[start//3:end//3]
@@ -419,8 +436,8 @@ def genome_mode(reference, reads, start, end, compress):
     names[idd] = name
     restore_codons(og_seqs, names)
     if len(err) > 0:
-        print('The following '+str(len(err))+' sequences were suspected of'
-              'containing frameshifts or an early stop codon and so were'
+        print('The following '+str(len(err))+' sequence(s) were suspected of '
+              'containing frameshifts or an early stop codon and so were '
               'thrown out before multiple alignment:')
         for e in err:
             print(e)
@@ -449,14 +466,15 @@ def gene_mode(reference, reads, compress):
             og_seqs[record.id] = record.seq
     records = [SeqRecord(seqs[0], id=ids[0], description=names[0])]
     if compress:
-        seqs, names, ids, og_seqs = compressor(seqs[1:], names[1:], ids[1:], og_seqs)
+        seqs, names, ids, og_seqs = compressor(seqs[1:], names[1:], ids[1:],
+                                               og_seqs)
     combine_align(records, ids, names, seqs)
     new_names = dict(zip(ids, names))
     new_names[records[0].id] = records[0].description
     restore_codons(og_seqs, new_names)
     if len(err) > 0:
-        print('The following '+str(len(err))+' sequences were suspected of'
-              'containing frameshifts or an early stop codon and so were'
+        print('The following '+str(len(err))+' sequence(s) were suspected of '
+              'containing frameshifts or an early stop codon and so were '
               'thrown out before multiple alignment:')
         for e in err:
             print(e)
@@ -489,8 +507,8 @@ def mixed_mode(reference, reads, compress):
     names[idd] = name
     restore_codons(og_seqs, names)
     if len(err) > 0:
-        print('The following '+str(len(err))+' sequences were suspected of'
-              'containing frameshifts or an early stop codon and so were'
+        print('The following '+str(len(err))+' sequence(s) were suspected of '
+              'containing frameshifts or an early stop codon and so were '
               'thrown out before multiple alignment:')
         for e in err:
             print(e)
